CMD_LIST = {}
PikaAsst = {}
CMD_HELP = {}
Pika_Cmd = {}
INT_PLUG = ""
LOAD_PLUG = {}
COUNT_MSG = 0
USERS = {}
COUNT_PM = {}
LASTMSG = {}
const = {}
ISAFK = False
AFKREASON = None
pikalog = getLogger(__name__)
CHROME_DRIVER = "/usr/bin/chromedriver"
GOOGLE_CHROME_BIN = "/usr/bin/google-chrome"
BOTLOG_CHATID = os.environ.get("BOTLOG_CHATID", None)
BF_BOT = os.environ.get("TG_BOT_TOKEN_BF_HER")
BF_BOTNAME = os.environ.get("TG_BOT_USER_NAME_BF_HER")
# Db initialisation



#!/usr/bin/env python3
#
# Copyright (C) 2020 by ItzSjDude@Github, < https://github.com/ItzSjDude/PikachuUserbot >.
#
# This file is part of < https://github.com/ItzSjDude/PikachuUserbot > project,
# and is released under the "GNU v3.0 License Agreement".
#
# Please see < https://github.com/ItzSjDude/PikachuUserbot/blob/master/LICENSE >
#
# All rights reserved

from urllib.parse import urlparse
from sqlalchemy import *
from rejson import Client, Path
import os

import os
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session
_get=os.environ.get

# the secret configuration specific things

def start() -> scoped_session:
    engine = create_engine(_get("DATABASE_URL"))
    BASE.metadata.bind=engine
    BASE.metadata.create_all(engine)
    return scoped_session(sessionmaker(bind=engine, autoflush=False))


try:
    BASE=declarative_base()
    SESSION=start()
except AttributeError as e:
    # this is a dirty way for the work-around required for #23
    print("DB_URI is not configured. Features depending on the database might have issues.")
    print(str(e))


class BotUsers(BASE):
    __tablename__="botusers"
    pika_id=Column(String(14), primary_key=True)

    def __init__(self, pika_id):
        self.pika_id=pika_id


class PikaChats(BASE):
    __tablename__="PikaTg"
    pika_id=Column(String(14), primary_key=True)

    def __init__(self, pika_id):
        self.pika_id=pika_id


class GMute(BASE):
    __tablename__="gmute"
    sender=Column(String(14), primary_key=True)
    pika_id=Column(Numeric, primary_key=True)

    def __init__(self, sender, pika_id):
        self.sender=str(sender)
        self.pika_id=pika_id


class GBan(BASE):
    __tablename__="gban"
    sender=Column(String(14), primary_key=True)
    pika_id=Column(Numeric, primary_key=True)
    reason=Column(UnicodeText)

    def __init__(self, sender, pika_id, reason=""):
        self.sender=str(sender)
        self.pika_id=str(pika_id)
        self.reason=reason


class Mute(BASE):
    __tablename__="mute"
    sender=Column(String(14), primary_key=True)
    chat_id=Column(String(14), primary_key=True)
    pika_id=Column(Numeric, primary_key=True)

    def __init__(self, sender, chat_id, pika_id):
        self.sender=str(sender)
        self.chat_id=str(chat_id)
        self.pika_id=pika_id


class Notes(BASE):
    __tablename__="notes"
    chat_id=Column(String(14), primary_key=True)
    keyword=Column(UnicodeText, primary_key=True, nullable=False)
    reply=Column(UnicodeText)
    f_mesg_id=Column(Numeric)
    client_id=Column(Numeric, primary_key=True)

    def __init__(self, chat_id, keyword, reply, f_mesg_id, client_id):
        self.chat_id=str(chat_id)
        self.keyword=keyword
        self.reply=reply
        self.f_mesg_id=f_mesg_id
        self.client_id=client_id


class PMPermit(BASE):
    __tablename__="pmpermit"
    chat_id=Column(String(14), primary_key=True)
    reason=Column(String(127))
    pika_id=Column(Numeric, primary_key=True)

    def __init__(self, chat_id, pika_id, reason=""):
        self.chat_id=chat_id
        self.reason=reason
        self.pika_id=pika_id


class Welcome(BASE):
    __tablename__="welcome"
    chat_id=Column(String(14), primary_key=True)
    pika_id=Column(Numeric, primary_key=True)
    cust_wc=Column(UnicodeText)
    cl_wc=Column(Boolean, default=False)
    prev_wc=Column(BigInteger)
    mf_id=Column(UnicodeText)

    def __init__(self, chat_id, pika_id, cust_wc, cl_wc, prev_wc, mf_id=None):
        self.chat_id=chat_id
        self.pika_id=pika_id
        self.cust_wc=cust_wc
        self.cl_wc=cl_wc
        self.prev_wc=prev_wc
        self.mf_id=mf_id


Mute.__table__.create(checkfirst=True)
BotUsers.__table__.create(checkfirst=True)
PikaChats.__table__.create(checkfirst=True)
GMute.__table__.create(checkfirst=True)
GBan.__table__.create(checkfirst=True)
Notes.__table__.create(checkfirst=True)
PMPermit.__table__.create(checkfirst=True)
Welcome.__table__.create(checkfirst=True)


def add_welcome(chat_id, pika_id, cust_wc, cl_wc, prev_wc, mf_id):
    add_wc=Welcome(chat_id, pika_id, cust_wc, cl_wc, prev_wc, mf_id)
    SESSION.add(add_wc)
    SESSION.commit()


def remove_welcome(chat_id, pika_id):
    rm_wc=SESSION.query(Welcome).get((str(chat_id), pika_id))
    if rm_wc:
        SESSION.delete(rm_wc)
        SESSION.commit()


def upd_prev_welcome(chat_id, pika_id, prev_wc):
    _update=SESSION.query(Welcome).get((str(chat_id), pika_id))
    _update.prev_wc=prev_wc
    SESSION.commit()


def get_welcome(chat_id, pika_id):
    try:
        return SESSION.query(Welcome).get((str(chat_id), pika_id))
    except Exception as e:
        pikalog.error(str(e))
        return
    finally:
        SESSION.close()


def clean_welcome(chat_id, pika_id, cl_wc):
    clnn=SESSION.query(Welcome).get((str(chat_id), pika_id))
    clnn.cl_wc=cl_wc
    SESSION.commit()


def approve(chat_id, pika_id, reason):
    adder=PMPermit(str(chat_id), pika_id, str(reason))
    SESSION.add(adder)
    SESSION.commit()


def disapprove(chat_id, pika_id):
    rem=SESSION.query(PMPermit).get((str(chat_id), pika_id))
    if rem:
        SESSION.delete(rem)
        SESSION.commit()


def get_all_approved(pika_id):
    rem=SESSION.query(PMPermit).filter(PMPermit.pika_id == pika_id).all()
    SESSION.close()
    return rem


def is_approved(chat_id, pika_id):
    try:
        return SESSION.query(PMPermit).filter(
            PMPermit.chat_id == str(chat_id),
            PMPermit.pika_id == pika_id).one()
    except BaseException:
        return None
    finally:
        SESSION.close()


def get_note(chat_id, keyword, client_id):
    try:
        return SESSION.query(Notes).get((str(chat_id), keyword, client_id))
    finally:
        SESSION.close()


def get_notes(chat_id, client_id):
    try:
        return SESSION.query(Notes).filter(
            Notes.chat_id == str(chat_id),
            Notes.client_id == client_id).all()
    except BaseException:
        return None
    finally:
        SESSION.close()


def add_note(chat_id, keyword, reply, f_mesg_id, client_id):
    to_check=get_note(chat_id, keyword, client_id)
    if not to_check:
        adder=Notes(str(chat_id), keyword, reply, f_mesg_id, client_id)
        SESSION.add(adder)
        SESSION.commit()
        return True
    else:
        rem=SESSION.query(Notes).get((str(chat_id), keyword, client_id))
        SESSION.delete(rem)
        SESSION.commit()
        adder=Notes(str(chat_id), keyword, reply, f_mesg_id, client_id)
        SESSION.add(adder)
        SESSION.commit()
        return False


def rm_note(chat_id, keyword, client_id):
    to_check=get_note(chat_id, keyword, client_id)
    if not to_check:
        return False
    else:
        rem=SESSION.query(Notes).get((str(chat_id), keyword, client_id))
        SESSION.delete(rem)
        SESSION.commit()
        return True


def is_muted(sender, chat_id, pika_id):
    user=SESSION.query(Mute).get((str(sender), str(chat_id), pika_id))
    if user:
        return True
    else:
        return False


def mute(sender, chat_id, pika_id):
    adder=Mute(str(sender), str(chat_id), pika_id)
    SESSION.add(adder)
    SESSION.commit()


def unmute(sender, chat_id, pika_id):
    rem=SESSION.query(Mute).get((str(sender), str(chat_id), pika_id))
    if rem:
        SESSION.delete(rem)
        SESSION.commit()


def get_all_muted(pika_id):
    rem=SESSION.query(Mute).filter(Notes.pika_id == pika_id).all()
    SESSION.close()
    return rem


def is_gbanned(sender, pika_id):
    try:
        _pikaG=SESSION.query(GBan).get((str(sender), str(pika_id)))
        if _pikaG:
            return str(_pikaG.reason)
    finally:
        SESSION.close()


def gban(sender, pika_id, reason):
    adder=GBan(str(sender), str(pika_id), str(reason))
    SESSION.add(adder)
    SESSION.commit()


def ungban(sender, pika_id):
    rem=SESSION.query(GBan).get((str(sender), str(pika_id)))
    if rem:
        SESSION.delete(rem)
        SESSION.commit()


def is_gmuted(sender):
    try:
        return SESSION.query(GMute).all()
    except BaseException:
        return None
    finally:
        SESSION.close()


def gmute(sender, pika_id):
    adder=GMute(str(sender), pika_id)
    SESSION.add(adder)
    SESSION.commit()


def ungmute(sender, pika_id):
    rem=SESSION.query(GMute).get((str(sender), pika_id))
    if rem:
        SESSION.delete(rem)
        SESSION.commit()


def add_pika(pika_id):
    pika=PikaChats(str(pika_id))
    SESSION.add(pika)
    SESSION.commit()


def is_pika_exist(pika_id):
    try:
        pika=SESSION.query(PikaChats).filter(
            PikaChats.pika_id == str(pika_id)).one()
        if pika:
            return True
    except BaseException:
        return None
    finally:
        SESSION.close()


def get_pika_chats():
    try:
        pika=SESSION.query(PikaChats).all()
        if pika:
            return pika
    except BaseException:
        return None
    finally:
        SESSION.close()


def add_user(pika_id: int):
    pika=BotUsers(str(pika_id))
    SESSION.add(pika)
    SESSION.commit()


def is_user_exist(pika_id):
    try:
        pika=SESSION.query(BotUsers).filter(
            BotUsers.pika_id == str(pika_id)).one()
        if pika:
            return True
    except BaseException:
        return None
    finally:
        SESSION.close()


def get_added_users():
    pika=SESSION.query(BotUsers).all()
    SESSION.close()
    return pika


url=urlparse(os.environ.get("REDIS_URL"))
host=url.host
port=url.port
username=url.username
password=url.password
pikadb=Client(
    host=host,
    port=port,
    username=username,
    password=password,
    decode_responses=True)
pikadb.jsonset=pikaset
pikadb.jsondel=pikadel
pikadb.jsonget=pikaget
pikadb.jsonarrappend=pikaseta
pikadb.jsonarrpop=pikageta
cdata={
    'main': {
        'session': None,
        'pmsecurity': False,
        'pmmsg': None,
        'alive_Name': None,
        'alivepic': None,
        'alivemsg': None,
        'cmdhandler': None,
        'pmlogger': None,
        'auser': None,
        'dauser': None,
        'pluginchannel': None
    },
    'multi1': {
        'session': None,
        'pmsecurity': False,
        'pmmsg': None,
        'alive_Name': None,
        'alivepic': None,
        'alivemsg': None,
        'pmlogger': None,
        'auser': None,
        'dauser': None,
        'pluginchannel': None
    },
    'multi2': {
        'session': None,
        'pmsecurity': False,
        'pmmsg': None,
        'alive_Name': None,
        'alivepic': None,
        'alivemsg': None,
        'pmlogger': None,
        'auser': None,
        'dauser': None
    },
    'multi3': {
        'session': None,
        'Gdrive': None,
        'Pmsecurity': False,
        'pmmsg': None,
        'alive_Name': None,
        'alivepic': None,
        'alivemsg': None,
        'pmlogger': None,
        'auser': None,
        'dauser': None
    },
    'apis': {
        'youtubeapi': None,
        'lyndiaapi': None,
        'screenshotapi': None
    },
}


def startdb():
    old_db_exists=pikaget('cdata', Path.rootPath())
    if not old_db_exists:
        pikaset("cdata", Path.rootPath(), cdata)
    else:
        return


def pset(rejsclt, data, value):
    return pikaset('cdata', Path("{rejsclt.data}"), value)


def pget(rejsclt, data):
    return pikaget('cdata', Path(f"{rejsclt.data}"))


def pdel(rejsclt, value):
    return pikadel('cdata', Path(f"{rejsclt.data}"))



#!/usr/bin/env python3
#
# Copyright (C) 2020 by ItzSjDude@Github, < https://github.com/ItzSjDude/PikachuUserbot >.
#
# This file is part of < https://github.com/ItzSjDude/PikachuUserbot > project,
# and is released under the "GNU v3.0 License Agreement".
#
# Please see < https://github.com/ItzSjDude/PikachuUserbot/blob/master/LICENSE >
#
# All rights reserved
from . import *


async def init_pika():
    if bot is None:
        pikalog.info(
            "**MAINCLIENT**: Started Login Assistent, Do /start at {}'s PM".format(BF_BOTNAME))
        await pika_login("main")
    else:
        _const = {}
        _start = True
        from pikabot.login import pika_login, pika_msg
        _logstr_ = "__{}__: Connected 🔥"
        _logstr2_ = "__{}__: Started Login Assistent, Do /start at {}'s PM".format(
            _const, pdb.bf_uname)
        import glob
        path = './plugins/*.py'
        _path = './pikabot/Assistant/plugins/*.py'
        files = glob.glob(path)
        _files = glob.glob(_path)

        if _start:
            await tgbot.start()
            tgbot.me = await tgbot.get_me()
            tgbot.PikaAsst = {}
            tgbot.uid = telethon.utils.get_peer_id(tgbot.me)
            pikalog.info(_logstr_.format("TGBOT"))
            msg = _logstr_.format("_TGBOT_") + '\n\n'
            _logpika = await tgbot.send_message(pdb.botlog_chat, msg)
            if bot:
                try:
                    await bot.start()
                    cli1 = await client.get_messages(Client, None, filter=InputMessagesFilterDocument)
                    total = int(cli1.total)
                    total_doxx = range(0, total)
                    for ixo in total_doxx:
                        mxo = cli1[ixo].id
                        await client.download_media(await bot.get_messages(Client, ids=mxo), "pikabot/main_plugs")

                    bot.me = await bot.get_me()
                    bot.pika_cmd = {}
                    bot.uid = telethon.utils.get_peer_id(bot.me)
                    pikalog.info(_logstr_.format("MAINCLIENT"))
                    msg += _logstr_.format("MAINCLIENT") + "\n\n"
                    await pika_msg(_logpika, msg)
                except Exception as e:
                    pikalog.info(str(e))
                    pikalog.info(_logstr2_.format("MAINCLIENT"))
                    msg += _logstr2_.format("MAINCLIENT") + "\n\n"
                    await pika_msg(_logpika, msg)
                    await pika_login("main")

            if bot2:
                try:
                    await bot2.start()
                    pikalog.info(_logstr_.format("MULTICLIENT1"))
                    bot2.me = await bot2.get_me()
                    bot2.uid = telethon.utils.get_peer_id(bot2.me)
                    msg += _logstr_.format("MULTICLIENT1") + "\n\n"
                    await pika_msg(_logpika, msg)
                except BaseException:
                    pikalog.info(_logstr2_.format("MULTICLIENT1"))
                    msg += _logstr2_.format("MULTICLIENT1") + "\n\n"
                    await pika_msg(_logpika, msg)
                    await pika_login("multi1")
            if bot3:
                try:
                    await bot3.start()
                    pikalog.info(_logstr2_.format("MULTICLIENT2"))
                    bot3.me = await bot3.get_me()
                    bot3.uid = telethon.utils.get_peer_id(bot3.me)
                    msg += _logstr_.format("MULTICLIENT2") + "\n\n"
                    await pika_msg(_logpika, msg)
                except BaseException:
                    pikalog.info(_logstr2_.format("MULTICLIENT2"))
                    msg += _logstr2_.format("MULTICLIENT2") + "\n\n"
                    await pika_msg(_logpika, msg)
                    await pika_login("multi2")
            if bot4:
                try:
                    await bot4.start()
                    pikalog.info(_logstr_.format("MULTICLIENT3"))
                    bot4.me = await bot4.get_me()
                    bot4.uid = telethon.utils.get_peer_id(bot4.me)
                    msg += logstr_.format("MULTICLIENT3") + "\n\n"
                    await pika_msg(_logpika, msg)
                except BaseException:
                    pikalog.info(_logstr2_.format("MULTICLIENT3"))
                    msg += _logstr2_.format("MULTICLIENT3") + "\n\n"
                    await pika_msg(_logpika, msg)
                    await pika_login("multi3")

            if pdb.maincl and bot is None:
                pikalog.info(
                    "**MAINCLIENT**: Session Expired / Invalid, Started Login Assistent, Do /start at {}'s PM".format(BF_BOTNAME))
                await pika_login("main")

            if pdb.multicl1 and bot2 is None:
                pikalog.info(
                    "**MULTICLIENT1**: Session Expired / Invalid, Started Login Assistent, Do /start at {}'s PM".format(BF_BOTNAME))
                await pika_login("multi1")

            if pdb.multicl2 and bot3 is None:
                pikalog.info(
                    "**MULTICLIENT2**: Session Expired / Invalid, Started Login Assistent, Do /start at {}'s PM".format(BF_BOTNAME))
                await pika_login("multi2")

            if pdb.multicl3 and bot4 is None:
                pikalog.info(
                    "**MULTICLIENT3**: Session Expired, Started Login Assistent, Do /start at {}'s PM".format(BF_BOTNAME))
                await pika_login("multi3")

            def __load_plugs__():
                for name in files:
                    with open(name) as f:
                        path1 = Path(f.name)
                        shortname = path1.stem
                        pika_plugins(shortname.replace(".py", ""))

                for name in _files:
                    with open(name) as f:
                        _asstpath = Path(f.name)
                        shortname = _asstpath.stem
                        pika_assistant(shortname.replace(".py", ""))
            __load_plugs__()

            msg += "Sucessfully Loaded Plugins\n\n"
            await pika_msg(_logpika, msg)
            msg += "**Pikabot Started Sucessfully**"
            await pika_msg(_logpika, msg)

        if len(argv) not in (1, 3, 4):
            if client:
                await bot.disconnect()
            if client2:
                await bot2.disconnect()
            if client3:
                await bot3.disconnect()
            if client4:
                await bot4.disconnect()
        else:
            if bot:
                await client.run_until_disconnected()
            if bot2:
                await client2.run_until_disconnected()
            if bot3:
                await client3.run_until_disconnected()
            if bot4:
                await client4.run_until_disconnected()


if __name__ == "__main__":
    pikaloop = asyncio.get_event_loop()
    pikaloop.run_until_complete(init_pika())



from ..database import *


class pdb(object):
     api_id = _get("API_ID")
     api_hash = _get("API_HASH", None)
     bf_token = _get("TG_BOT_TOKEN_BF_HER", None)
     bf_uname = _get("TG_BOT_USER_NAME_BF_HER(
     maincl=pget("main", "session")
     multicl1=pget("multi1", "session")
     multicl2=pget("multi2", "session")
     multicl3=pget("multi3", "session")
